@@ -2654,8 +2654,8 @@ importance.AICzeroinfl <- function(cand.set, parm, modnames = NULL, second.ord =
 
 ##function for nicer printing of importance values
 print.importance <- function(x, digits = 2, ...) {
-  cat("\nImportance values of '", x$parm, "' :\n\n")
-  cat("w+ (models including parameter): ", round(x$w.plus, digits = digits), "\n")
-  cat("w- (models excluding parameter): ", round(x$w.minus, digits = digits), "\n")
+  cat("\nImportance values of '", x$parm, "':\n\n", sep = "")
+  cat("w+ (models including parameter):", round(x$w.plus, digits = digits), "\n")
+  cat("w- (models excluding parameter):", round(x$w.minus, digits = digits), "\n")
   cat("\n")
 }
